@@ -3,9 +3,10 @@
 
 namespace Fm {
 
-FileInfoJob::FileInfoJob(FilePathList paths, FilePath commonDirPath, const std::shared_ptr<const HashSet>& cutFilesHashSet):
+FileInfoJob::FileInfoJob(FilePathList paths, FilePathList deletionPaths, FilePath commonDirPath, const std::shared_ptr<const HashSet>& cutFilesHashSet):
     Job(),
     paths_{std::move(paths)},
+    deletionPaths_{std::move(deletionPaths)},
     commonDirPath_{std::move(commonDirPath)},
     cutFilesHashSet_{cutFilesHashSet} {
 }
@@ -15,12 +16,13 @@ void FileInfoJob::exec() {
         if(!isCancelled()) {
             GErrorPtr err;
             GFileInfoPtr inf{
-                g_file_query_info(path.gfile().get(), gfile_info_query_attribs,
+                g_file_query_info(path.gfile().get(), defaultGFileInfoQueryAttribs,
                                   G_FILE_QUERY_INFO_NONE, cancellable().get(), &err),
                 false
             };
-            if(!inf)
-                return;
+            if(!inf) {
+                continue;
+            }
 
             // Reuse the same dirPath object when the path remains the same (optimize for files in the same dir)
             auto dirPath = commonDirPath_.isValid() ? commonDirPath_ : path.parent();
