@@ -27,6 +27,7 @@
 #include "utilities.h"
 #include "ui_file-operation-dialog.h"
 
+
 namespace Fm {
 
 FileOperationDialog::FileOperationDialog(FileOperation* _operation):
@@ -41,35 +42,35 @@ FileOperationDialog::FileOperationDialog(FileOperation* _operation):
     QString title;
     QString message;
     switch(_operation->type()) {
-    case FM_FILE_OP_MOVE:
+    case FileOperation::Move:
         title = tr("Move files");
         message = tr("Moving the following files to destination folder:");
         break;
-    case FM_FILE_OP_COPY:
+    case FileOperation::Copy:
         title = tr("Copy Files");
         message = tr("Copying the following files to destination folder:");
         break;
-    case FM_FILE_OP_TRASH:
+    case FileOperation::Trash:
         title = tr("Trash Files");
         message = tr("Moving the following files to trash can:");
         break;
-    case FM_FILE_OP_DELETE:
+    case FileOperation::Delete:
         title = tr("Delete Files");
         message = tr("Deleting the following files:");
         ui->dest->hide();
         ui->destLabel->hide();
         break;
-    case FM_FILE_OP_LINK:
+    case FileOperation::Link:
         title = tr("Create Symlinks");
         message = tr("Creating symlinks for the following files:");
         break;
-    case FM_FILE_OP_CHANGE_ATTR:
+    case FileOperation::ChangeAttr:
         title = tr("Change Attributes");
         message = tr("Changing attributes of the following files:");
         ui->dest->hide();
         ui->destLabel->hide();
         break;
-    case FM_FILE_OP_UNTRASH:
+    case FileOperation::UnTrash:
         title = tr("Restore Trashed Files");
         message = tr("Restoring the following files from trash can:");
         ui->dest->hide();
@@ -100,47 +101,53 @@ int FileOperationDialog::ask(QString /*question*/, char* const* /*options*/) {
     return 0;
 }
 
-int FileOperationDialog::askRename(FmFileInfo* src, FmFileInfo* dest, QString& new_name) {
-    int ret;
+
+FileOperationJob::FileExistsAction FileOperationDialog::askRename(const FileInfo &src, const FileInfo &dest, FilePath &newDest) {
+    FileOperationJob::FileExistsAction ret;
     if(defaultOption == -1) { // default action is not set, ask the user
         RenameDialog dlg(src, dest, this);
         dlg.exec();
         switch(dlg.action()) {
         case RenameDialog::ActionOverwrite:
-            ret = FM_FILE_OP_OVERWRITE;
+            ret = FileOperationJob::OVERWRITE;
             if(dlg.applyToAll()) {
                 defaultOption = ret;
             }
             break;
-        case RenameDialog::ActionRename:
-            ret = FM_FILE_OP_RENAME;
-            new_name = dlg.newName();
+        case RenameDialog::ActionRename: {
+            ret = FileOperationJob::RENAME;
+            auto newName = dlg.newName();
+            if(!newName.isEmpty()) {
+                auto destDirPath = dest.path().parent();
+                newDest = destDirPath.child(newName.toUtf8().constData());
+            }
             break;
+        }
         case RenameDialog::ActionIgnore:
-            ret = FM_FILE_OP_SKIP;
+            ret = FileOperationJob::SKIP;
             if(dlg.applyToAll()) {
                 defaultOption = ret;
             }
             break;
         default:
-            ret = FM_FILE_OP_CANCEL;
+            ret = FileOperationJob::CANCEL;
             break;
         }
     }
     else {
-        ret = defaultOption;
+        ret = (FileOperationJob::FileExistsAction)defaultOption;
     }
     return ret;
 }
 
-FmJobErrorAction FileOperationDialog::error(GError* err, FmJobErrorSeverity severity) {
-    if(severity >= FM_JOB_ERROR_MODERATE) {
-        if(severity == FM_JOB_ERROR_CRITICAL) {
+Job::ErrorAction FileOperationDialog::error(GError* err, Job::ErrorSeverity severity) {
+    if(severity >= Job::ErrorSeverity::MODERATE) {
+        if(severity == Job::ErrorSeverity::CRITICAL) {
             QMessageBox::critical(this, tr("Error"), QString::fromUtf8(err->message));
-            return FM_JOB_ABORT;
+            return Job::ErrorAction::ABORT;
         }
         if (ignoreNonCriticalErrors_) {
-            return FM_JOB_CONTINUE;
+            return Job::ErrorAction::CONTINUE;
         }
         QMessageBox::StandardButton stb = QMessageBox::critical(this, tr("Error"), QString::fromUtf8(err->message),
                                                                 QMessageBox::Ok | QMessageBox::Ignore);
@@ -148,7 +155,7 @@ FmJobErrorAction FileOperationDialog::error(GError* err, FmJobErrorSeverity seve
             ignoreNonCriticalErrors_ = true;
         }
     }
-    return FM_JOB_CONTINUE;
+    return Job::ErrorAction::CONTINUE;
 }
 
 void FileOperationDialog::setCurFile(QString cur_file) {
@@ -156,15 +163,22 @@ void FileOperationDialog::setCurFile(QString cur_file) {
 }
 
 void FileOperationDialog::setDataTransferred(uint64_t finishedSize, std::uint64_t totalSize) {
-    ui->dataTransferred->setText(QString("%1 / %2")
+    ui->filesProcessed->setText(QString("%1 / %2")
                                  .arg(formatFileSize(finishedSize, fm_config->si_unit))
                                  .arg(formatFileSize(totalSize, fm_config->si_unit)));
 }
 
+void FileOperationDialog::setFilesProcessed(uint64_t finishedCount, uint64_t totalCount) {
+    ui->filesProcessed->setText(QString("%1 / %2")
+                                 .arg(finishedCount)
+                                 .arg(totalCount));
+}
+
 void FileOperationDialog::setPercent(unsigned int percent) {
     ui->progressBar->setValue(percent);
 }
 
+
 void FileOperationDialog::setRemainingTime(unsigned int sec) {
     unsigned int min = 0;
     unsigned int hr = 0;
