@@ -19,7 +19,6 @@
 
 
 #include "placesmodel.h"
-#include "icontheme.h"
 #include <gio/gio.h>
 #include <QDebug>
 #include <QMimeData>
@@ -55,25 +54,19 @@ PlacesModel::PlacesModel(QObject* parent):
 
     createTrashItem();
 
-    // FIXME: add an option to hide network:///
-    if(true) {
-        computerItem = new PlacesModelItem("computer", tr("Computer"), Fm::FilePath::fromUri("computer:///"));
-        placesRoot->appendRow(computerItem);
-    }
-    else {
-        computerItem = nullptr;
+    computerItem = new PlacesModelItem("computer", tr("Computer"), Fm::FilePath::fromUri("computer:///"));
+    placesRoot->appendRow(computerItem);
+
+    { // Applications
+        const char* applicaion_icon_names[] = {"system-software-install", "applications-accessories", "application-x-executable"};
+        // NOTE: g_themed_icon_new_from_names() accepts char**, but actually const char** is OK.
+        Fm::GIconPtr gicon{g_themed_icon_new_from_names((char**)applicaion_icon_names, G_N_ELEMENTS(applicaion_icon_names)), false};
+        auto fmicon = Fm::IconInfo::fromGIcon(std::move(gicon));
+        applicationsItem = new PlacesModelItem(fmicon, tr("Applications"), Fm::FilePath::fromUri("menu:///applications/"));
+        placesRoot->appendRow(applicationsItem);
     }
 
-    // FIXME: add an option to hide applications:///
-    const char* applicaion_icon_names[] = {"system-software-install", "applications-accessories", "application-x-executable"};
-    // NOTE: g_themed_icon_new_from_names() accepts char**, but actually const char** is OK.
-    Fm::GIconPtr gicon{g_themed_icon_new_from_names((char**)applicaion_icon_names, G_N_ELEMENTS(applicaion_icon_names)), false};
-    auto fmicon = Fm::IconInfo::fromGIcon(std::move(gicon));
-    applicationsItem = new PlacesModelItem(fmicon, tr("Applications"), Fm::FilePath::fromUri("menu:///applications/"));
-    placesRoot->appendRow(applicationsItem);
-
-    // FIXME: add an option to hide network:///
-    if(true) {
+    { // Network
         const char* network_icon_names[] = {"network", "folder-network", "folder"};
         // NOTE: g_themed_icon_new_from_names() accepts char**, but actually const char** is OK.
         Fm::GIconPtr gicon{g_themed_icon_new_from_names((char**)network_icon_names, G_N_ELEMENTS(network_icon_names)), false};
@@ -81,9 +74,6 @@ PlacesModel::PlacesModel(QObject* parent):
         networkItem = new PlacesModelItem(fmicon, tr("Network"), Fm::FilePath::fromUri("network:///"));
         placesRoot->appendRow(networkItem);
     }
-    else {
-        networkItem = nullptr;
-    }
 
     devicesRoot = new QStandardItem(tr("Devices"));
     devicesRoot->setSelectable(false);
@@ -171,7 +161,7 @@ PlacesModel::~PlacesModel() {
         g_object_unref(trashMonitor_);
     }
 
-    Q_FOREACH(GMount* mount, shadowedMounts_) {
+    for(GMount* const mount : qAsConst(shadowedMounts_)) {
         g_object_unref(mount);
     }
 }
@@ -429,6 +419,18 @@ void PlacesModel::onMountRemoved(GVolumeMonitor* monitor, GMount* mount, PlacesM
 }
 
 void PlacesModel::onVolumeAdded(GVolumeMonitor* /*monitor*/, GVolume* volume, PlacesModel* pThis) {
+    // the item may have been added with "mount-added" (as in loopback mounting)
+    bool itemExists = false;
+    GMount* mount = g_volume_get_mount(volume);
+    if(mount) {
+        if(pThis->itemFromMount(mount)) {
+            itemExists = true;
+        }
+        g_object_unref(mount);
+    }
+    if(itemExists) {
+        return;
+    }
     // for some unknown reasons, sometimes we get repeated volume-added
     // signals and added a device more than one. So, make a sanity check here.
     PlacesModelVolumeItem* volumeItem = pThis->itemFromVolume(volume);
